/*
 * Copyright 2020 Netflix, Inc.
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package com.netflix.conductor.core.execution.mapper;

import com.netflix.conductor.common.metadata.tasks.Task;
import com.netflix.conductor.common.metadata.tasks.TaskDef;
import com.netflix.conductor.common.metadata.workflow.WorkflowTask;
import com.netflix.conductor.common.run.Workflow;
import com.netflix.conductor.core.execution.ParametersUtils;
import com.netflix.conductor.dao.MetadataDAO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

public class JsonJQTransformTaskMapper implements TaskMapper {

    public static final Logger LOGGER = LoggerFactory.getLogger(JsonJQTransformTaskMapper.class);
    private final ParametersUtils parametersUtils;
    private final MetadataDAO metadataDAO;

    public JsonJQTransformTaskMapper(ParametersUtils parametersUtils, MetadataDAO metadataDAO) {
        this.parametersUtils = parametersUtils;
        this.metadataDAO = metadataDAO;
    }

    @Override
    public List<Task> getMappedTasks(TaskMapperContext taskMapperContext) {

        LOGGER.debug("TaskMapperContext {} in JsonJQTransformTaskMapper", taskMapperContext);

        WorkflowTask taskToSchedule = taskMapperContext.getTaskToSchedule();
        Workflow workflowInstance = taskMapperContext.getWorkflowInstance();
        String taskId = taskMapperContext.getTaskId();

        TaskDef taskDefinition = Optional.ofNullable(taskMapperContext.getTaskDefinition())
                .orElseGet(() -> Optional.ofNullable(metadataDAO.getTaskDef(taskToSchedule.getName()))
                        .orElse(null));

        Map<String, Object> taskInput = parametersUtils
                .getTaskInputV2(taskToSchedule.getInputParameters(), workflowInstance, taskId, taskDefinition);

        Task jsonJQTransformTask = new Task();
        jsonJQTransformTask.setTaskType(taskToSchedule.getType());
        jsonJQTransformTask.setTaskDefName(taskToSchedule.getName());
        jsonJQTransformTask.setReferenceTaskName(taskToSchedule.getTaskReferenceName());
        jsonJQTransformTask.setWorkflowInstanceId(workflowInstance.getWorkflowId());
        jsonJQTransformTask.setWorkflowType(workflowInstance.getWorkflowName());
        jsonJQTransformTask.setCorrelationId(workflowInstance.getCorrelationId());
        jsonJQTransformTask.setStartTime(System.currentTimeMillis());
        jsonJQTransformTask.setScheduledTime(System.currentTimeMillis());
        jsonJQTransformTask.setInputData(taskInput);
        jsonJQTransformTask.setTaskId(taskId);
        jsonJQTransformTask.setStatus(Task.Status.IN_PROGRESS);
        jsonJQTransformTask.setWorkflowTask(taskToSchedule);
        jsonJQTransformTask.setWorkflowPriority(workflowInstance.getPriority());

        return Collections.singletonList(jsonJQTransformTask);
    }
}
